@@ -107,6 +107,12 @@
     case ')': \
     case ALPHANUMERIC
     
+#define NUMBER_TERMINATOR \
+    ',': \
+    case WHITESPACE: \
+    case ']': \
+    case '}': \
+    case '/'
 
 static const int HEX_MULT[] = {4096, 256, 16, 1};
 
@@ -331,21 +337,27 @@ enum LaxJsonError lax_json_feed(struct LaxJsonContext *context, int size, const
                     case '/':
                     case '\\':
                         BUFFER_CHAR(c);
+                        context->state = LaxJsonStateString;
                         break;
                     case 'b':
                         BUFFER_CHAR('\b');
+                        context->state = LaxJsonStateString;
                         break;
                     case 'f':
                         BUFFER_CHAR('\f');
+                        context->state = LaxJsonStateString;
                         break;
                     case 'n':
                         BUFFER_CHAR('\n');
+                        context->state = LaxJsonStateString;
                         break;
                     case 'r':
                         BUFFER_CHAR('\r');
+                        context->state = LaxJsonStateString;
                         break;
                     case 't':
                         BUFFER_CHAR('\t');
+                        context->state = LaxJsonStateString;
                         break;
                     case 'u':
                         context->state = LaxJsonStateUnicodeEscape;
@@ -589,11 +601,7 @@ enum LaxJsonError lax_json_feed(struct LaxJsonContext *context, int size, const
                         BUFFER_CHAR(c);
                         context->state = LaxJsonStateNumberDecimal;
                         break;
-                    case ',':
-                    case WHITESPACE:
-                    case ']':
-                    case '}':
-                    case '/':
+                    case NUMBER_TERMINATOR:
                         BUFFER_CHAR('\0');
                         if (context->number(context, atof(context->value_buffer)))
                             return LaxJsonErrorAborted;
@@ -617,6 +625,12 @@ enum LaxJsonError lax_json_feed(struct LaxJsonContext *context, int size, const
                         BUFFER_CHAR('e');
                         context->state = LaxJsonStateNumberExponentSign;
                         break;
+                    case NUMBER_TERMINATOR:
+                        context->state = LaxJsonStateNumber;
+                        /* rewind 1 */
+                        data -= 1;
+                        context->column -= 1;
+                        break;
                     default:
                         return LaxJsonErrorUnexpectedChar;
                 }
@@ -709,3 +723,20 @@ enum LaxJsonError lax_json_eof(struct LaxJsonContext *context) {
         }
     }
 }
+
+const char *lax_json_str_err(enum LaxJsonError err) {
+    switch (err) {
+        case LaxJsonErrorNone: return "none";
+        case LaxJsonErrorUnexpectedChar: return "unexpected character";
+        case LaxJsonErrorExpectedEof: return "expected end of file";
+        case LaxJsonErrorExceededMaxStack: return "exceeded max stack";
+        case LaxJsonErrorNoMem: return "out of memory";
+        case LaxJsonErrorExceededMaxValueSize: return "exceeded maximum value size";
+        case LaxJsonErrorInvalidHexDigit: return "invalid hex digit";
+        case LaxJsonErrorInvalidUnicodePoint: return "invalid unicode point";
+        case LaxJsonErrorExpectedColon: return "expected colon";
+        case LaxJsonErrorUnexpectedEof: return "unexpected end of file";
+        case LaxJsonErrorAborted: return "aborted";
+    }
+    return "invalid error code";
+}
